@@ -24,21 +24,21 @@ stsm_init_vals = function(y, par, freq, trend, decomp = "", seasons = NULL, prio
   sp = stsm_ssm(par, y, decomp, trend)
   init = list(B0 = sp$B0, P0 = sp$P0)
   if("trend" %in% colnames(prior)){
-    init[["B0"]][names(init[["B0"]]) == "Tt0"] = prior[!is.na(trend), ]$trend[1]
+    init[["B0"]][names(init[["B0"]]) == "Tt_0"] = prior[!is.na(trend), ]$trend[1]
   }
   if("cycle" %in% colnames(prior)){
-    init[["B0"]][names(init[["B0"]]) %in% c("Ct", "Cts")] = prior[!is.na(cycle), ]$cycle[1]
+    init[["B0"]][names(init[["B0"]]) %in% c("Ct_0", "Cts_0")] = prior[!is.na(cycle), ]$cycle[1]
   }
   if("drift" %in% colnames(prior)){
-    init[["B0"]][names(init[["B0"]]) == "Mt0"] = prior[!is.na(drift), ]$drift[1]
+    init[["B0"]][names(init[["B0"]]) == "Dt_0"] = prior[!is.na(drift), ]$drift[1]
   }
   if("remainder" %in% colnames(prior)){
-    init[["B0"]][names(init[["B0"]]) == "et0"] = prior[!is.na(remainder), ]$remainder[1]
+    init[["B0"]][names(init[["B0"]]) == "et_0"] = prior[!is.na(remainder), ]$remainder[1]
   }
   if(grepl("seasonal", decomp)){
     if(!is.null(seasons)){
       for(j in seasons){
-        init[["B0"]][names(init[["B0"]]) %in% paste0(c("St", "Sts"), j)] = 
+        init[["B0"]][names(init[["B0"]]) %in% paste0(c("St", "Sts"), j, "_0")] = 
           prior[!is.na(eval(parse(text = paste0("seasonal", j)))), c(paste0("seasonal", j)), with = FALSE][[1]][1]
       }
     }
